@@ -4,8 +4,8 @@
 #' surrogate and true endpoints (Stijven et al., 2022). Because the bivariate
 #' distributions of the surrogate-true endpoint pairs are functionally
 #' independent across treatment groups, a bivariate distribution is fitted in
-#' each treatment group separately. The marginal distributions are based on the Royston-Parmar
-#' survival model (Royston and Parmar, 2002).
+#' each treatment group separately. The marginal distributions are based on the
+#' Royston-Parmar survival model (Royston and Parmar, 2002).
 #'
 #' @details
 #'
@@ -13,13 +13,13 @@
 #'
 #' In the causal-inference approach to evaluating surrogate endpoints, the first
 #' step is to estimate the joint distribution of the relevant potential
-#' outcomes. Let \eqn{(T_0, S_0, S_1, T_1)'} denote the vector of potential
+#' outcomes. Let \eqn{(T_0, S_0, S_1, T_1)'}.  denote the vector of potential
 #' outcomes where \eqn{(S_k, T_k)'} is the pair of potential outcomes under
 #' treatment \eqn{Z = k}. \eqn{T} refers to the true endpoint, e.g., overall
 #' survival. \eqn{S} refers to the composite surrogate endpoint, e.g.,
-#' progression-free-survival. Because \eqn{S} is
-#' usually a composite endpoint with death as possible event, modeling
-#' difficulties arise because \eqn{Pr(S_k = T_k) > 0}.
+#' progression-free-survival. Because \eqn{S} is usually a composite endpoint
+#' with death as possible event, modeling difficulties arise because \eqn{Pr(S_k
+#' = T_k) > 0}.
 #'
 #' Due to difficulties in modeling the composite surrogate and the true endpoint
 #' jointly, the time-to-surrogate event (\eqn{\tilde{S}}) is modeled instead of
@@ -35,8 +35,8 @@
 #' Two modelling choices are made before estimating the two bivariate
 #' distributions described in the previous paragraph:
 #' * The number of internal knots for the Royston-Parmar survival models. This
-#' is specified through the `n_knots` argument. The number of knots is assumed to
-#' be equal across the four margins.
+#' is specified through the `n_knots` argument. The number of knots is assumed
+#' to be equal across the four margins.
 #' * The parametric family of the bivariate copulas. The parametric family is
 #' assumed to be equal across treatment groups. This choice is specified through
 #' the `copula_family` argument.
@@ -60,20 +60,25 @@
 #' death.
 #'
 #' @references Stijven, F., Alonso, a., Molenberghs, G., Van Der Elst, W., Van
-#' Keilegom, I. (2022). An information-theoretic approach to the evaluation of
-#' time-to-event surrogates for time-to-event true endpoints based on causal
-#' inference.
+#'   Keilegom, I. (2022). An information-theoretic approach to the evaluation of
+#'   time-to-event surrogates for time-to-event true endpoints based on causal
+#'   inference.
 #'
-#' Royston, P., & Parmar, M. K. (2002). Flexible parametric proportional-hazards
-#' and proportional-odds models for censored survival data, with application to
-#' prognostic modelling and estimation of treatment effects. Statistics in
-#' medicine, 21(15), 2175-2197.
+#'   Royston, P., & Parmar, M. K. (2002). Flexible parametric
+#'   proportional-hazards and proportional-odds models for censored survival
+#'   data, with application to prognostic modelling and estimation of treatment
+#'   effects. Statistics in medicine, 21(15), 2175-2197.
 #'
 #'
 #' @param data A data frame in the correct format (See details).
-#' @param copula_family One of the following parametric copula families:
-#'   `"clayton"`, `"frank"`, `"gaussian"`, or `"gumbel"`.
-#' @param n_knots Number of internal knots for the Royston-Parmar survival model.
+#' @param copula_family  One of the following parametric copula families:
+#'   `"clayton"`, `"frank"`, `"gaussian"`, or `"gumbel"`. The first element in
+#'   `copula_family` corresponds to the control group, the second to the
+#'   experimental group.
+#' @param n_knots Number of internal knots for the Royston-Parmar survival
+#'   models for \eqn{\tilde{S}_0}, \eqn{T_0}, \eqn{\tilde{S}_1}, and \eqn{T_1}.
+#'   If `length(n_knots) == 1`, the same number of knots are assumed for the
+#'   four marginal distributions.
 #' @param fitted_model Fitted model from which initial values are extracted. If
 #'   `NULL` (default), standard initial values are used. This option intended
 #'   for when a model is repeatedly fitted, e.g., in a bootstrap.
@@ -88,7 +93,7 @@
 #'
 #' @author Florian Stijven
 #'
-#' @seealso [marginal_gof_scr()], [sensitivity_analysis_SurvSurv_copula()]
+#' @seealso [sensitivity_analysis_SurvSurv_copula()]
 #'
 #' @examples
 #' if(require(Surrogate)) {
@@ -112,6 +117,14 @@ fit_model_SurvSurv = function(data,
                               fitted_model = NULL,
                               method = "BFGS",
                               maxit = 500) {
+  # If copula_family is length 1, we repeat the same copula family.
+  if (length(copula_family) == 1) {
+    copula_family = rep(copula_family, 2)
+  }
+  # If the length of n_knots is 1, increase the length to 4 by repeating the original element.
+  if (length(n_knots == 1)) {
+    n_knots = rep(n_knots, 4)
+  }
   # Column names are added to make the intrepretation of the further code
   # easier. Pfs refers to the surrogate, Surv refers to the true endpoint.
   colnames(data) = c("Pfs", "Surv", "Treat", "PfsInd", "SurvInd")
@@ -125,20 +138,20 @@ fit_model_SurvSurv = function(data,
   # gives the final estimate.
   twostep_fit0 = twostep_SurvSurv(
     X = data0$Pfs,
-    delta_X = data0$PfsInd,
+    delta_X = ifelse(data0$Surv == data0$Pfs & data0$SurvInd == 1, 1, data0$PfsInd),
     Y = data0$Surv,
     delta_Y = data0$SurvInd,
-    copula_family = copula_family,
-    n_knots = n_knots,
+    copula_family = copula_family[1],
+    n_knots = n_knots[1:2],
     method = method
   )
   twostep_fit1 = twostep_SurvSurv(
     X = data1$Pfs,
-    delta_X = data1$PfsInd,
+    delta_X = ifelse(data1$Surv == data1$Pfs & data1$SurvInd == 1, 1, data1$PfsInd),
     Y = data1$Surv,
     delta_Y = data1$SurvInd,
-    copula_family = copula_family,
-    n_knots = n_knots,
+    copula_family = copula_family[2],
+    n_knots = n_knots[3:4],
     method = method
   )
 
@@ -167,7 +180,7 @@ fit_model_SurvSurv = function(data,
       delta_X = data0$PfsInd,
       Y = data0$Surv,
       delta_Y = data0$SurvInd,
-      copula_family = copula_family,
+      copula_family = copula_family[1],
       knotsx = knots0,
       knotsy = knott0
     )
@@ -192,7 +205,7 @@ fit_model_SurvSurv = function(data,
       delta_X = data1$PfsInd,
       Y = data1$Surv,
       delta_Y = data1$SurvInd,
-      copula_family = copula_family,
+      copula_family = copula_family[2],
       knotsx = knots1,
       knotsy = knott1
     )
@@ -209,6 +222,15 @@ fit_model_SurvSurv = function(data,
     )
   })
 
+  # We fit survival copulas in the above functions. This corresponds to 180
+  # degree rotated copulas.
+  copula_rotations = c(180, 180)
+  # The Gaussian copula is invariant to rotations and a non-zero rotation
+  # parameter for the Gaussian copula will give errors. The rotation parameters
+  # are therefore set to zero for the Gaussian copula. Equivalently for the
+  # Frank copula.
+  copula_rotations = ifelse(copula_family %in% c("gaussian", "frank"), 0, copula_rotations)
+
 
   return(
     new_vine_copula_ss_fit(
@@ -218,7 +240,9 @@ fit_model_SurvSurv = function(data,
       knots0 = knots0,
       knots1 = knots1,
       knott0 = knott0,
-      knott1 = knott1
+      knott1 = knott1,
+      copula_rotations = copula_rotations,
+      data = data
     )
   )
 }
@@ -233,13 +257,22 @@ fit_model_SurvSurv = function(data,
 #' @param knots1 placement of knots for Royston-Parmar model
 #' @param knott0 placement of knots for Royston-Parmar model
 #' @param knott1 placement of knots for Royston-Parmar model
+#' @param copula_rotations vector of copula rotation parameters
+#' @param data Original data
 #'
 #' @return S3 object
 #'
 #' @examples
 #' #should not be used be the user
-new_vine_copula_ss_fit = function(fit_0, fit_1, copula_family,
-                                  knots0, knots1, knott0, knott1){
+new_vine_copula_ss_fit = function(fit_0,
+                                  fit_1,
+                                  copula_family,
+                                  knots0,
+                                  knots1,
+                                  knott0,
+                                  knott1,
+                                  copula_rotations,
+                                  data) {
   structure(
     .Data = list(
       fit_0 = fit_0,
@@ -248,7 +281,9 @@ new_vine_copula_ss_fit = function(fit_0, fit_1, copula_family,
       knots0 = knots0,
       knots1 = knots1,
       knott0 = knott0,
-      knott1 = knott1
+      knott1 = knott1,
+      copula_rotations = copula_rotations,
+      data = data
     ),
     class = "vine_copula_SurvSurv_fit"
   )
@@ -300,9 +335,9 @@ model_fit_measures = function(fitted_model){
   copula_par1 = coef(fitted_model$fit_1)[2*(n_knots + 2) + 1]
   #convert fitted copula parameters to kendall's tau scale
   tau_0 = conversion_copula_tau(copula_par = copula_par0,
-                                copula_family = fitted_model$copula_family)
+                                copula_family = fitted_model$copula_family[1])
   tau_1 = conversion_copula_tau(copula_par = copula_par1,
-                                copula_family = fitted_model$copula_family)
+                                copula_family = fitted_model$copula_family[2])
 
   #compute total maximized log likelihood
   log_lik = fitted_model$log_lik0 + fitted_model$log_lik1
@@ -384,15 +419,15 @@ twostep_SurvSurv = function(X,
   copula_start = starting_values_list$inv_tau
   start = c(marg_coef_x, marg_coef_y, copula_start)
   # Add informative names to the starting values.
-  names(start) = c(paste(names(start)[1:(n_knots + 2)], "(S)"),
-                   paste(names(start)[1:(n_knots + 2)], "(T)"),
+  names(start) = c(paste(names(start)[1:(n_knots[1] + 2)], "(S)"),
+                   paste(names(start)[(n_knots[1] + 3):(sum(n_knots) + 4)], "(T)"),
                    "theta (copula)")
   suppressWarnings({
     ml_fit = maxLik::maxLik(
       logLik = log_lik_function,
       start = start,
       method = method,
-      fixed = 1:(2 * n_knots + 4)
+      fixed = 1:(sum(n_knots) + 4)
     )
   })
 
@@ -415,19 +450,20 @@ survival_survival_loglik =  function(para,
                                      delta_Y,
                                      copula_family,
                                      knotsx,
-                                     knotsy)
+                                     knotsy,
+                                     sum_observations = TRUE)
 {
-  k = length(knotsx) - 2
   switch(
     copula_family,
-    "clayton" = clayton_loglik(para, X, Y, delta_X, delta_Y, k, knotsx, knotsy),
-    "frank" = frank_loglik(para, X, Y, delta_X, delta_Y, k, knotsx, knotsy),
-    "gumbel" = gumbel_loglik(para, X, Y, delta_X, delta_Y, k, knotsx, knotsy),
-    "gaussian" = normal_loglik(para, X, Y, delta_X, delta_Y, k, knotsx, knotsy),
+    "clayton" = clayton_loglik(para, X, Y, delta_X, delta_Y, knotsx, knotsy, sum_observations),
+    "frank" = frank_loglik(para, X, Y, delta_X, delta_Y, knotsx, knotsy, sum_observations),
+    "gumbel" = gumbel_loglik(para, X, Y, delta_X, delta_Y, knotsx, knotsy, sum_observations),
+    "gaussian" = normal_loglik(para, X, Y, delta_X, delta_Y, knotsx, knotsy, sum_observations),
   )
 }
 
 SurvSurv_starting_values = function(X, delta_X, Y, delta_Y, copula_family, n_knots){
+  requireNamespace("copula")
   # The starting value for the association parameter is obtained by estimating
   # the copula parameter through Kendall's tau, ignoring censoring. The
   # estimated Kendall's tau is then converted to the copula parameter scale.
@@ -461,13 +497,13 @@ SurvSurv_starting_values = function(X, delta_X, Y, delta_Y, copula_family, n_kno
   fit_s = flexsurv::flexsurvspline(
     formula = survival::Surv(X, delta_X) ~ 1,
     data = data,
-    k = n_knots,
+    k = n_knots[1],
     scale = "hazard"
   )
   fit_t = flexsurv::flexsurvspline(
     formula = survival::Surv(Y, delta_Y) ~ 1,
     data = data,
-    k = n_knots,
+    k = n_knots[2],
     scale = "hazard"
   )
 
@@ -487,3 +523,23 @@ print.vine_copula_SurvSurv_fit = function(x, ...) {
   print(summary(x$fit_1))
 }
 
+plot.vine_copula_SurvSurv_fit = function(x, composite = TRUE, grid = NULL, time_unit = "years", ...) {
+  if (is.null(grid)) {
+    # If the user did not provide a grid, compute the default grid.
+    grid = seq(from = 1e-5,
+               to = max(x$data[, 1:2]),
+               length.out = 2e2)
+  }
+  # Plot marginal survival functions.
+  if (composite)
+    marginal_gof_plots_scr(fitted_model = x, grid = grid)
+  # Plot expected progression time as a function of OS for the subpopulation of
+  # patients that progress before dying.
+  mean_S_before_T_plots_scr(fitted_model = x,
+                            grid = grid)
+  # Plot the probability of not having progressed before dying, given the
+  # survival time.
+  prob_dying_without_progression_plots(fitted_model = x,
+                         grid = grid)
+}
+
