@@ -1,18 +1,7 @@
 test_that("sensitivity_analysis_BinCont_copula() works in a simplified setting",
           {
-            copula_family = "clayton"
-            marginal_surrogate = "normal"
-            data("Schizo_BinCont")
-            na = is.na(Schizo_BinCont$CGI_Bin) | is.na(Schizo_BinCont$PANSS)
-            X = Schizo_BinCont$PANSS[!na]
-            Y = Schizo_BinCont$CGI_Bin[!na]
-            Treat = Schizo_BinCont$Treat[!na]
-            Treat = ifelse(Treat == 1, 1, 0)
-            data = data.frame(X,
-                              Y,
-                              Treat)
-            fitted_model = fit_copula_model_BinCont(data, copula_family, marginal_surrogate, twostep = FALSE)
-
+            # Load fitted copula model.
+            fitted_model = readRDS(test_path("fixtures", "schizo-dvine-clayton.rds"))
             n_sim = 2
             set.seed(1)
             sens_results = sensitivity_analysis_BinCont_copula(
@@ -32,18 +21,8 @@ test_that("sensitivity_analysis_BinCont_copula() works in a simplified setting",
 
 test_that("sensitivity_analysis_BinCont_copula() works in a simplified setting with limits in the sensitivity analysis",
           {
-            copula_family = "clayton"
-            marginal_surrogate = "normal"
-            data("Schizo_BinCont")
-            na = is.na(Schizo_BinCont$CGI_Bin) | is.na(Schizo_BinCont$PANSS)
-            X = Schizo_BinCont$PANSS[!na]
-            Y = Schizo_BinCont$CGI_Bin[!na]
-            Treat = Schizo_BinCont$Treat[!na]
-            Treat = ifelse(Treat == 1, 1, 0)
-            data = data.frame(X,
-                              Y,
-                              Treat)
-            fitted_model = fit_copula_model_BinCont(data, copula_family, marginal_surrogate, twostep = FALSE)
+            # Load fitted copula model.
+            fitted_model = readRDS(test_path("fixtures", "schizo-dvine-clayton.rds"))
 
             n_sim = 2
             set.seed(1)
